import asyncio
import aiohttp

class RulingsObject(object):
	def __init__(self, _url, **kwargs):
		self.pretty = kwargs.get('pretty')
		self.format = kwargs.get('format')
		self._url = 'https://api.scryfall.com/' + _url
		loop = asyncio.get_event_loop()
		self.session = aiohttp.ClientSession(loop=loop)

		async def getRequest(url, **kwargs):
			async with self.session.get(url, **kwargs) as response:
				return await response.json()

		self.scryfallJson = loop.run_until_complete(getRequest(
			url = self._url,
			params={
				'format': self.format,
				'pretty': self.pretty
			}))

		if self.scryfallJson['object'] == 'error':
			self.session.close()
			raise Exception(self.scryfallJson['details'])

		self.session.close()

	def __checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except KeyError:
			return None

	def __checkForTupleKey(self, parent, num, key):
		try:
			return self.scryfallJson[parent][num][key]
		except KeyError:
			return None

	def object(self):
		if self.__checkForKey('object') is None:
			return KeyError('This ruling object has no associated object key.')

		return self.scryfallJson['object']

	def has_more(self):
		if self.__checkForKey('has_more') is None:
			return KeyError('This ruling object has no associated has_more key.')

		return self.scryfallJson['has_more']

	def data(self):
		if self.__checkForKey('data') is None:
			return KeyError('This ruling object has no associated data key.')

		return self.scryfallJson['data']

	def data_length(self):
		if self.__checkForKey('data') is None:
			return KeyError('This ruling object has no associated data key.')

		return len(self.scryfallJson['data'])

	def ruling_object(self, num):
		if self.__checkForTupleKey('data', num, 'object') is None:
			return KeyError('This ruling has no object key.')

		return self.scryfallJson['data'][num]['object']

	def ruling_source(self, num):
		if self.__checkForTupleKey('data', num, 'source') is None:
			return KeyError('This ruling has no source key.')

		return self.scryfallJson['data'][num]['source']

	def ruling_published_at(self, num):
		if self.__checkForTupleKey('data', num, 'published_at') is None:
			return KeyError('This ruling has no published_at key.')

		return self.scryfallJson['data'][num]['published_at']

	def ruling_comment(self, num):
		if self.__checkForTupleKey('data', num, 'comment') is None:
			return KeyError('This ruling has no comment key.')

		return self.scryfallJson['data'][num]['comment']



import aiohttp
import asyncio

class ScryfallObject(object):
	def __init__(self, _url, **kwargs):
		self._url = 'https://api.scryfall.com/' + _url
		loop = asyncio.get_event_loop()
		self.session = aiohttp.ClientSession(loop=loop)

		async def getRequest(url, **kwargs):
			async with self.session.get(url, **kwargs) as response:
				return await response.json()

		self.scryfallJson = loop.run_until_complete(getRequest(
			url = self._url,
			params={
				'format': kwargs.get('format'),
				'face': kwargs.get('face'),
				'version': kwargs.get('version'),
				'pretty': kwargs.get('pretty')
			}))

		if self.scryfallJson['object'] == 'error':
			self.session.close()
			raise Exception(self.scryfallJson['details'])

		self.session.close()

	def __checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except KeyError:
			return None

	def object(self):
		if self.__checkForKey('object') is None:
			return KeyError("This card has no associated object key.")

		return self.scryfallJson['object']

	def id(self):
		if self.__checkForKey('id') is None:
			return KeyError("This card has no associated id key.")

		return self.scryfallJson['id']

	def multiverse_ids(self):
		if self.__checkForKey('multiverse_ids') is None:
			return KeyError("This card has no associated multiverse id key.")

		return self.scryfallJson['multiverse_ids']

	def mtgo_id(self):
		if self.__checkForKey('mtgo_id') is None:
			return KeyError("This card has no associated mtgo id key.")

		return self.scryfallJson['mtgo_id']

	def mtgo_foil_id(self):
		if self.__checkForKey('mtgo_foil_id') is None:
			return KeyError("This card has no associate mtgo foil id key.")

		return self.scryfallJson['mtgo_foil_id']

	def name(self):
		if self.__checkForKey('name') is None:
			return KeyError("This card has no associated name key.")

		return self.scryfallJson['name']

	def uri(self):
		if self.__checkForKey('uri') is None:
			return KeyError("This card has no associated uri key.")

		return self.scryfallJson['uri']

	def scryfall_uri(self):
		if self.__checkForKey('scryfall_uri') is None:
			return KeyError("This card has no associated scryfall uri key.")

		return self.scryfallJson['scryfall_uri']

	def layout(self):
		if self.__checkForKey('layout') is None:
			return KeyError("This card has no associated layout key.")

		return self.scryfallJson['layout']

	def highres_image(self):
		if self.__checkForKey('highres_image') is None:
			return KeyError("This card has no associated highres image key.")

		return self.scryfallJson['highres_image']

	def image_uris(self):
		if self.__checkForKey('image_uris') is None:
			return KeyError("This card has no associated image uris key.")

		return self.scryfallJson['image_uris']

	def cmc(self):
		if self.__checkForKey('cmc') is None:
			return KeyError("This card has no associated cmc key.")

		return self.scryfallJson['cmc']

	def type_line(self):
		if self.__checkForKey('type_line') is None:
			return KeyError("This card has no associated type line key.")

		return self.scryfallJson['type_line']

	def oracle_text(self):
		if self.__checkForKey('oracle_text') is None:
			return KeyError("This card has no associated oracle text key.")

		return self.scryfallJson['oracle_text']

	def mana_cost(self):
		if self.__checkForKey('mana_cost') is None:
			return KeyError("This card has no associated mana cost key.")

		return self.scryfallJson['mana_cost']

	def colors(self):
		if self.__checkForKey('colors') is None:
			return KeyError("This card has no associated colors key.")

		return self.scryfallJson['colors']

	def color_identity(self):
		if self.__checkForKey('color_identity') is None:
			return KeyError("This card has no associated color identity key.")

		return self.scryfallJson['color_identity']

	def legalities(self):
		if self.__checkForKey('legalities') is None:
			return KeyError("This card has no associated legalities key.")

		return self.scryfallJson['legalities']

	def reserved(self):
		if self.__checkForKey('reserved') is None:
			return KeyError("This card has no associated  reserved key key.")

		return self.scryfallJson['reserved']

	def reprint(self):
		if self.__checkForKey('reprint') is None:
			return KeyError("This card has no associated reprint key.")

		return self.scryfallJson['reprint']

	def set(self):
		if self.__checkForKey('set') is None:
			return KeyError("This card has no associated set key.")

		return self.scryfallJson['set']

	def set_name(self):
		if self.__checkForKey('set_name') is None:
			return KeyError("This card has no associated set name key.")

		return self.scryfallJson['set_name']

	def set_uri(self):
		if self.__checkForKey('set_uri') is None:
			return KeyError("This card has no associated set uri key.")

		return self.scryfallJson['set_uri']

	def set_search_uri(self):
		if self.__checkForKey('set_search_uri') is None:
			return KeyError("This card has no associated set search uri key.")

		return self.scryfallJson['set_search_uri']

	def scryfall_set_uri(self):
		if self.__checkForKey('scryfall_set_uri') is None:
			return KeyError("This card has no associated scryfall set uri key.")

		return self.scryfallJson['scryfall_set_uri']

	def rulings_uri(self):
		if self.__checkForKey('rulings_uri') is None:
			return KeyError("This card has no associated rulings uri key.")

		return self.scryfallJson['rulings_uri']

	def prints_search_uri(self):
		if self.__checkForKey('prints_search_uri') is None:
			return KeyError("This card has no associated prints search uri key.")

		return self.scryfallJson['prints_search_uri']

	def collector_number(self):
		if self.__checkForKey('collector_number') is None:
			return KeyError("This card has no associated collector number key.")

		return self.scryfallJson['collector_number']

	def digital(self):
		if self.__checkForKey('digital') is None:
			return KeyError("This card has no associated digital key key.")

		return self.scryfallJson['digital']

	def rarity(self):
		if self.__checkForKey('rarity') is None:
			return KeyError("This card has no associated rarity key.")

		return self.scryfallJson['rarity']

	def illustration_id(self):
		if self.__checkForKey('illustration_id') is None:
			return KeyError("This card has no associated illustration id key.")

		return self.scryfallJson['illustration_id']

	def artist(self):
		if self.__checkForKey('artist') is None:
			return KeyError("This card has no associated artist key.")

		return self.scryfallJson['artist']

	def frame(self):
		if self.__checkForKey('frame') is None:
			return KeyError("This card has no associated frame key.")

		return self.scryfallJson['frame']

	def full_art(self):
		if self.__checkForKey('') is None:
			return KeyError("This card has no associated full art key key.")

		return self.scryfallJson['full_art']

	def border_color(self):
		if self.__checkForKey('border_color') is None:
			return KeyError("This card has no associated border color key.")

		return self.scryfallJson['border_color']

	def timeshifted(self):
		if self.__checkForKey('timeshifted') is None:
			return KeyError("This card has no associated timeshifted key key.")

		return self.scryfallJson['timeshifted']

	def colorshifted(self):
		if self.__checkForKey('colorshifted') is None:
			return KeyError("This card has no associated colorshifted key key.")

		return self.scryfallJson['colorshifted']

	def futureshifted(self):
		if self.__checkForKey('futureshifted') is None:
			return KeyError("This card has no associated futureshifted key key.")

		return self.scryfallJson['futureshifted']

	def edhrec_rank(self):
		if self.__checkForKey('edhrec_rank') is None:
			return KeyError("This card has no associated edhrec rank key.")

		return self.scryfallJson['edhrec_rank']

	def currency(self, mode):
		modes = ['usd', 'eur', 'tix']
		if mode not in modes:
			return KeyError("{} is not a key.".format(mode))

		if self.__checkForKey(mode) is None:
			return KeyError("This card has no associated currency {} key".format(mode))

		return self.scryfallJson[mode]

	def related_uris(self):
		if self.__checkForKey('related_uris') is None:
			return KeyError("This card has no associated related uris key.")

		return self.scryfallJson['related_uris']

	def purchase_uris(self):
		if self.__checkForKey('purchase_uris') is None:
			return KeyError("This card has no associated purchase uris key.")

		return self.scryfallJson['purchase_uris']

	def life_modifier(self):
		if self.__checkForKey('life_modifier') is None:
			return KeyError("This card has no associated life modifier key.")

		return self.scryfallJson['life_modifier']

	def hand_modifier(self):
		if self.__checkForKey('hand_modifier') is None:
			return KeyError("This card has no associated hand modifier key.")

		return self.scryfallJson['hand_modifier']

	def color_indicator(self):
		if self.__checkForKey('color_indicator') is None:
			return KeyError("This card has no associated color indicator key.")

		return self.scryfallJson['color_indicator']

	def all_parts(self):
		if self.__checkForKey('all_parts') is None:
			return KeyError("This card has no associated all parts key.")

		return self.scryfallJson['all_parts']

	def card_faces(self):
		if self.__checkForKey('card_faces') is None:
			return KeyError("This card has no associated card faces key.")

		return self.scryfallJson['card_faces']

	def watermark(self):
		if self.__checkForKey('watermark') is None:
			return KeyError("This card has no associated watermark key.")

		return self.scryfallJson['watermark']

	def story_spotlight_number(self):
		if self.__checkForKey('story_spotlight_number') is None:
			return KeyError("This card has no associated story spotlight number key.")

		return self.scryfallJson['story_spotlight_number']

	def story_spotlight_uri(self):
		if self.__checkForKey('story_spotlight_uri') is None:
			return KeyError("This card has no associated story spotlight uri key.")

		return self.scryfallJson['story_spotlight_uri']



from .autocomplete import Autocomplete
from .collector import Collector
from .cardid import Id
from .mtgo import Mtgo
from .multiverse import Multiverse
from .named import Named
from .randomcard import Random



from .scryfall_object import ScryfallObject

class Collector(ScryfallObject):
	""" cards/:code/:collector_number

	Parameters:
		code: str				The 3 or 4 letter set code.
		collector_number: int	The collector number of the card.
	"""

	def __init__(self, **kwargs):
		self.code = kwargs.get('code')
		self.collector_number = kwargs.get('collector_number')
		self.url = 'cards/{}/{}'.format(self.code, str(self.collector_number))
		super(Collector, self).__init__(self.url)



#Import classes from cards
from scrython.cards import Autocomplete
from scrython.cards import Collector
from scrython.cards import Id
from scrython.cards import Mtgo
from scrython.cards import Multiverse
from scrython.cards import Named
from scrython.cards import Random

#Import classes from rulings
from scrython.rulings import Mtgo
from scrython.rulings import Multiverse
from scrython.rulings import Id
from scrython.rulings import Code



from .scryfall_object import ScryfallObject
import urllib.parse

class Named(ScryfallObject):
	"""
	Parameters:
		exact: str				The exact card name to search for, case insenstive.
		fuzzy: str				A fuzzy card name to search for.
		set: str				A set code to limit the search to one set.
	"""

	def __init__(self, **kwargs):
		self.exact = kwargs.get('exact')
		self.fuzzy = kwargs.get('fuzzy')
		self.set = kwargs.get('set')
		self.dict = {}

		if self.exact is not None:
			self.dict['exact'] = self.exact

		if self.fuzzy is not None:
			self.dict['fuzzy'] = self.fuzzy

		if self.set is not None:
			self.dict['set'] = self.set

		self.args = urllib.parse.urlencode(self.dict)
		self.url = 'cards/named?' + self.args
		super(Named, self).__init__(self.url)



from .rulings_object import RulingsObject

class Id(RulingsObject):
    def __init__(self, **kwargs):
        self.id = str(kwargs.get('id'))
        self.url = 'cards/{}/rulings'.format(self.id)
        super(Id, self).__init__(self.url)



from .rulings_object import RulingsObject

class Mtgo(RulingsObject):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id')
        self.url = 'cards/mtgo/{}/rulings'.format(self.id)
        super(Mtgo, self).__init__(self.url)



import asyncio, aiohttp

class Autocomplete(object):
	""" cards/autocomplete

	Parameters:
		query: str		The string to autocomplete.
		format: str		The data format to return. Currently only supports JSON.
		pretty: bool	If true, the returned JSON will be prettified. Avoid using for production code.

	Attributes:
		object: str			Returns the type of object it is.
		total_items: int	Returns the number of items in data.
		data: arr			The full autocompleted list.

	"""

	def __init__(self, query, **kwargs):
		self.query = query
		self.pretty = kwargs.get('pretty')
		self.format = kwargs.get('format')
		loop = asyncio.get_event_loop()
		self.session = aiohttp.ClientSession(loop=loop)

		async def getRequest(url, **kwargs):
			async with self.session.get(url, **kwargs) as response:
				return await response.json()

		self.scryfallJson = loop.run_until_complete(getRequest(
			url='https://api.scryfall.com/cards/autocomplete?',
			params={'q':self.query, 'pretty':self.pretty, 'format':self.format}))

		if self.scryfallJson['object'] == 'error':
			self.session.close()
			raise Exception(self.scryfallJson['details'])

		self.session.close()

	def __checkForKey(self, key):
		try:
			return self.scryfallJson[key]
		except KeyError:
			return None

	def object(self):
		if self.__checkForKey('object') is None:
			return KeyError('This card has no associated object key.')

		return self.scryfallJson['object']

	def total_items(self):
		if self.__checkForKey('total_items') is None:
			return KeyError('This card has no associated total items key.')

		return self.scryfallJson['total_items']

	def data(self):
		if self.__checkForKey('data') is None:
			return KeyError('This card has no associated data key.')

		return self.scryfallJson['data']



from .scryfall_object import ScryfallObject

class Multiverse(ScryfallObject):
	"""
	Parameters:
		id: int		The multiverse id of the card.
	"""

	def __init__(self, **kwargs):
		self.multiverseid = kwargs.get('id')
		self.url = 'cards/multiverse/' + self.multiverseid
		super(Multiverse, self).__init__(self.url)



from .scryfall_object import ScryfallObject

class Id(ScryfallObject):
	""" cards/:id

	Parameters:
		id: str					The Scryfall id of the card.
	"""

	def __init__(self, **kwargs):
		self.id = kwargs.get('id')
		self.url = 'cards/' + str(self.id)
		super(Id, self).__init__(self.url)



from .scryfall_object import ScryfallObject


class Random(ScryfallObject):
    """This will return a random card. No parameters are passed while creating."""
    def __init__(self):
        self.url = 'cards/random'
        super(Random, self).__init__(self.url)



from .rulings_object import RulingsObject

class Code(RulingsObject):
    def __init__(self, code, collector_number):
        self.code = code.lower()
        self.number = str(collector_number)
        self.url = 'cards/{}/{}/rulings'.format(self.code, self.number)
        super(Code, self).__init__(self.url)



from .scryfall_object import ScryfallObject
import urllib.parse

class Mtgo(ScryfallObject):
	""" cards/mtgo/:id

	Parameters:
		id: int		The mtgo id of the card.

	"""

	def __init__(self, **kwargs):
		self.mtgoid = kwargs.get('id')
		self.url = 'cards/mtgo/' + str(self.mtgoid)
		super(Mtgo, self).__init__(self.url)
